package main

import (
	"encoding/hex"
	"errors"
	"fmt"
	"log"
	"net"
	"crypto/sha1"
	"strconv"
	"strings"
	"time"


	"im-microservice/helper"

	"golang.org/x/net/context"
	"google.golang.org/grpc"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/metadata"
)

const (
	port     = ":50051"
)


func checksum(req map[string]string) (error) {
	err,AppSecret := helper.GetSecretByKey(req["appkey"])
	if err != nil {
		return err
	}
	server_time := time.Now().Unix()
	client_time,err := strconv.Atoi(req["curtime"])
	if err != nil {
		return err
	}
	if server_time > int64(client_time) {
		return errors.New("当前客户端时间不能小于服务端时间")
	}

	sha11 := sha1.New()
	sha11.Write([]byte(AppSecret+req["nonce"] + req["curtime"]))
	sevice_checksum := hex.EncodeToString(sha11.Sum([]byte(nil)))
	if sevice_checksum != req["checksum"] {
		return errors.New("加密串不正确")
	}

	return nil
}

func auth(ctx context.Context, req interface{}, info *grpc.UnaryServerInfo, handler grpc.UnaryHandler) (interface{}, error) {

	_, ok := metadata.FromIncomingContext(ctx)
	if !ok {
		return nil, grpc.Errorf(codes.Unauthenticated, "无Token认证信息")
	}
	request_map := make(map[string]string)
	s := fmt.Sprintf("%v", req)
	reqs := strings.Fields(s)
	if len(reqs) > 0 {
		for _, v := range reqs {
			params := strings.Split(v, ":")
			request_map[params[0]] = params[1]
		}
	}
	if len(request_map) < 0 {
		return nil, errors.New("请求参数为空")
	}

	if _, ok := request_map["appkey"]; !ok {
		return nil, errors.New("缺少appkey")
	}
	if _, ok := request_map["nonce"]; !ok {
		return nil, errors.New("缺少nonce")
	}
	if _, ok := request_map["curtime"]; !ok {
		return nil, errors.New("缺少nonce")
	}
	if _, ok := request_map["checksum"]; !ok {
		return nil, errors.New("checksum")
	}
	if err := checksum(request_map); err != nil {
		return nil, err
	}

	return handler(ctx, req)
}

func main() {

	lis, err := net.Listen("tcp", port)
	if err != nil {
		log.Fatalf("failed to listen: %v", err)
	}

	var opts []grpc.ServerOption
	opts = append(opts, grpc.UnaryInterceptor(auth))
	s := grpc.NewServer(opts...)

	//pb.RegisterUserServer(s, &user.UserSevice{})
	//pb.RegisterAppServer(s, &app.AppSevice{})
	if err := s.Serve(lis); err != nil {
		log.Fatalf("failed to serve: %v", err)
	}

}
